(defun MoransI(rates adj &key mean print)
  "
;; Examples:
 (def rates '(62 12 89 4))
 (def adj (matrix (list 4 4) '(0 1 1 0 1 0 0 1 1 0 0 1 0 1 1 0)))
 (MoransI rates adj :mean 42 :print t)
 (def rates '(62 12 4 89))
 (MoransI rates adj :print t)
 (def rates '(4 12 62 89))
 (def adj (matrix (list 4 4) '(0 1 1 0 1 0 0 1 1 0 0 1 0 1 1 0)))
 (MoransI rates adj :print t)
"
  (let* (
	 (mean (if mean mean (mean rates)))
	 (z (- rates mean))
	 (N (length z))
	 (So (sum adj))
	 (bigsum (dot z (matmult  adj z)))
	 (MoransI (/
	      (* N bigsum) 
	      (* So (dot z z))
	      )
	     )
	 )
    (if print
	(let ()
	  (print 'adjacency)
	  (mprint adj)
	  (print 'data-locations)
	  (mprint (matrix (list (round (sqrt N)) (round (sqrt N))) rates))
	  (mprint 
	   (bind-columns
	    '(rates mean-rate z sum-sq-z N So bigsum MoransI)
	    (list rates mean z (sum (square z)) N So bigsum MoransI)
	    )
	   )
	  )
      )
    MoransI
    )
  )
(try-out MoransI)

(def simulation
     (mapcar 
      (lambda(n)
	(MoransI (sample rates (length rates)) adj :print t)
	)
      (iseq 100)
      )
     )

(def rates '(100 100 0 0))
(def adj (matrix (list 4 4) '(0 1 1 0 1 0 0 1 1 0 0 1 0 1 1 0)))
(MoransI rates adj)

(def rates '(1 1 0 0))
(MoransI rates adj)

(def rates '(1 0 0 1))
(MoransI rates adj)

